! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!
!  Definition of the F90 interface to the Data Subspace
!
!	AUTHOR:
!	  D. Hinshaw (DH)
!

module DSSinter

  use Dal

  implicit none

  private

  ! Dss handles

  type DssT
     integer, pointer :: ptr
  end type DssT

  type DScompT
     integer, pointer :: ptr
  end type DScompT

  type DSfilterT
     integer, pointer :: ptr
  end type DSfilterT

  public :: DssT, DScompT, DSfilterT

  ! Public procedures

  public :: dataSubspace
  public :: dssRead
  public :: dssWrite
  public :: dssRelease
  public :: dssAddComp
  public :: dssDeleteComponent
  public :: dssComponent
  public :: dssComponents
  public :: selectExpression
  public :: getDssRanges
  public :: releaseDssRanges
!*  public :: dssVolume  ! Not yet implemented
!*  public :: dssClean   ! Not yet implemented

  public :: dssDeleteFilter
  public :: dssFilter
  public :: dssHasFilter
  public :: dssFilters

  public :: dssFilterType
  public :: dssFilterName
  public :: dssFilterUnits
  public :: dssFilterComps
  public :: dssFilterBlock
  public :: dssFilterMask

  public :: dssMerge
  public :: dssDelete

  ! Interfaces

  interface dataSubspace
     module procedure newDataSubspace
     module procedure getDataSubspace
  end interface

  interface getDssRanges
     module procedure getSubspaceRanges
     module procedure getComponentRanges
     module procedure getFilterRanges
  end interface

  interface selectExpression
     module procedure subspaceSelectExpression
     module procedure compSelectExpression
     module procedure filterSelectExpression
  end interface

  interface dssFilter
     module procedure dssFilterbyNumber
     module procedure dssFilterbyName
  end interface

  ! Parameters

  character(len=*), parameter :: errorMessage  = "Invalid DSS library"

  ! Filter types

  integer, public, parameter :: RANGEFILTERT = 1
  integer, public, parameter :: REGIONFILTERT = 2
  integer, public, parameter :: BITMASKFILTERT = 3
  integer, public, parameter :: RANGEBLOCKFILTERT = 4
  integer, public, parameter :: GTIFILTERT = 5
  integer, public, parameter :: MASKFILTERT = 6
  integer, public, parameter :: REGIONBLOCKFILTERT = 7

contains

  function newDataSubspace()
    type(DssT) :: newDataSubspace
    
    call error( "", errorMessage )

  end function newDataSubspace
  

  function getDataSubspace( block )
    type(DssT) :: getDataSubspace
    type(BlockT), intent(in) :: block

    call error( "", errorMessage )

  end function getDataSubspace


  subroutine dssRead( outDss, inBlock )
    type(DssT), intent(in) :: outDss
    type(BlockT), intent(in) :: inBlock

    call error( "", errorMessage )

  end subroutine dssRead


  subroutine dssWrite( inDss, outBlock )
    type(DssT), intent(in) :: inDss
    type(BlockT), intent(in) :: outBlock

    call error( "", errorMessage )

  end subroutine dssWrite


  subroutine dssRelease( dss )
    type(DssT), intent(in) :: dss
    
    call error( "", errorMessage )

  end subroutine dssRelease


  function dssAddComp( dss )
    type(DScompT) :: dssAddComp
    type(DssT), intent(in) :: dss
    
    call error( "", errorMessage )

  end function dssAddComp


  subroutine dssDeleteComponent( dss, iComp )
    type(DssT), intent(in) :: dss
    integer, intent(in) :: iComp

    call error( "", errorMessage )

  end subroutine dssDeleteComponent


  function dssComponent( dss, iComp )
    type(DScompT) :: dssComponent
    type(DssT), intent(in) :: dss
    integer, intent(in) :: iComp

    call error( "", errorMessage )

  end function dssComponent


  function dssComponents( dss )
    integer :: dssComponents
    type(DssT), intent(in) :: dss

    call error( "", errorMessage )

  end function dssComponents


  function subspaceSelectExpression( dss, axes )
    character(len=1024) :: subspaceSelectExpression
    type(DssT), intent(in) :: dss
    character(len=*), intent(in), optional :: axes
    
    call error( "", errorMessage )

  end function subspaceSelectExpression


  function compSelectExpression( dscomp, axes )
    character(len=1024) :: compSelectExpression
    type(DScompT), intent(in) :: dscomp
    character(len=*), intent(in), optional :: axes
    
    call error( "", errorMessage )

  end function compSelectExpression


  function filterSelectExpression( dsfilter )
    character(len=1024) :: filterSelectExpression
    type(DSfilterT), intent(in) :: dsfilter
    
    call error( "", errorMessage )

  end function filterSelectExpression


  subroutine getSubspaceRanges( dss, axis, lowerB, lower, upper, upperB )
    type(DssT), intent(in) :: dss
    character(len=*), intent(in) :: axis
    integer, dimension(:), pointer :: lowerB, upperB
    real(kind=double), dimension(:), pointer :: lower, upper

    call error( "", errorMessage )

  end subroutine getSubspaceRanges

  
  subroutine getComponentRanges( dscomp, axis, lowerB, lower, upper, upperB )
    type(DScompT), intent(in) :: dscomp
    character(len=*), intent(in) :: axis
    integer, dimension(:), pointer :: lowerB, upperB
    real(kind=double), dimension(:), pointer :: lower, upper

    call error( "", errorMessage )

  end subroutine getComponentRanges


  subroutine getFilterRanges( dsfilter, lowerB, lower, upper, upperB )
    type(DSfilterT), intent(in) :: dsfilter
    integer, dimension(:), pointer :: lowerB, upperB
    real(kind=double), dimension(:), pointer :: lower, upper

    call error( "", errorMessage )

  end subroutine getFilterRanges


  subroutine releaseDssRanges( lowerB, lower, upper, upperB )
    integer, dimension(:), pointer :: lowerB, upperB
    real(kind=double), dimension(:), pointer :: lower, upper

    call error( "", errorMessage )

  end subroutine releaseDssRanges

  
  subroutine dssDeleteFilter( dscomp, filterNumber )
    type(DScompT), intent(in) :: dscomp
    integer, intent(in) :: filterNumber

    call error( "", errorMessage )

  end subroutine dssDeleteFilter


  function dssFilterbyNumber( dscomp, filterNumber )
    type(DSfilterT) :: dssFilterbyNumber
    type(DScompT), intent(in) :: dscomp
    integer, intent(in) :: filterNumber

    call error( "", errorMessage )

  end function dssFilterbyNumber


  function dssFilterbyName( dscomp, filterName, filterType )
    type(DSfilterT) dssFilterbyName
    type(DScompT), intent(in) :: dscomp
    character(len=*) filterName
    integer, intent(in) :: filterType

    call error( "", errorMessage )

  end function dssFilterbyName


  function dssHasFilter( dscomp, filterName, filterType )
    logical dssHasFilter
    type(DScompT), intent(in) :: dscomp
    character(len=*), intent(in) :: filterName
    integer, intent(in) :: filterType

    call error( "", errorMessage )

  end function dssHasFilter


  function dssFilters( dscomp )
    integer :: dssFilters
    type(DScompT), intent(in) :: dscomp

    call error( "", errorMessage )

  end function dssFilters


  function dssFilterType( filter )
    integer :: dssFilterType
    type(DSfilterT) :: filter

    call error( "", errorMessage )

  end function dssFilterType


  function dssFilterName( filter )
    character(len=80) :: dssFilterName
    type(DSfilterT) :: filter

    call error( "", errorMessage )

  end function dssFilterName


  function dssFilterUnits( filter )
    character(len=80) :: dssFilterUnits
    type(DSfilterT) :: filter

    call error( "", errorMessage )

  end function dssFilterUnits


  function dssFilterComps( filter )
    character(len=80) :: dssFilterComps
    type(DSfilterT) :: filter

    call error( "", errorMessage )

  end function dssFilterComps
    

  function dssFilterBlock( filter )
    character(len=255) :: dssFilterBlock
    type(DSfilterT) :: filter

    call error( "", errorMessage )

  end function dssFilterBlock


  subroutine dssFilterMask( filter, onBits, offBits )
    type(DSfilterT) :: filter
    integer(kind=int32), dimension(:), pointer :: onBits, offBits

    call error( "", errorMessage )

  end subroutine dssFilterMask

  
  subroutine dssMerge(dss1, dss2)
    type(DssT) :: dss1, dss2

    call error( "", errorMessage )

  end subroutine dssMerge


  subroutine dssDelete(block)
    type(BlockT) :: block

    call error( "", errorMessage )

  end subroutine dssDelete


end module DSSinter
