// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
// Authors:  Dean Hinshaw - University of Leicester
//  
// DSS test : Construct a DSS specification and test that it produces
//            a proper selectlib expression
//


#include <cstring>

#include "param.h"
#include <Dal.h>
#include <Region.h>
#include <DataSubspace.h>
#include <errstr.h>
#include <DataFilter.h>

int main(int argc, char **argv)
{
  if (argc != 2) {
    cerr<<"Usage: "<<argv[0]<<" <block> \n";
    return 1;
  }

  std::string block(argv[1]);
  DataSet * elist = dataSetServer->open(block, DataSet::Read);
  Block * eblk = elist->block(block);

  DataSubspace dss;

  eblk >> dss;

  errstr << "Equivalent selection expression for data subspace is " << endl << dss.selectExpression()
	 << Sas::message(Msg::App, Msg::Always);

  dataSetServer->close(elist);
  return 0;
}
  
