// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//---------------------------------------------------------------------------
//
//                     XMM Science Analysis System (SAS)
//                    (c) 1997-2010 European Space Agency
//
//---------------------------------------------------------------------------
//
//	testDssBuilder.cc: --- test harness for DSS builder
//
//	CHANGE HISTORY:
//		UL, 1999-11-16: 1.0.0: - first version
//
//	REFERENCES:
//		[1]: Documentation of selectlib package, U. Lammers
//
//	AUTHOR:
//		U. Lammers (UL), Tue Nov 16 09:56:55 MET 1999
//
//	$Header: /sasbuild/repositories/sasdev/sas/packages/dsslib/test/testDssBuilder.cc,v 1.1.1.1 2003/11/24 16:10:38 eojero Exp $
//

#include <cassert>
#include <string>
#include "DssBuilder.h"
#include "Evaluator.h"
#include "DataSubspace.h"
#include "errstr.h"

int main(int argc, char **argv)
{
	if (argc != 2) {
		cerr<<"Usage: "<<argv[0]<<" <expr> \n";
		return 1;
	}

	string expr(argv[1]);
	try {
		DataSubspace dss = dssBuilder.dss(expr);

		string dssexpr(dss.selectExpression());
		
		cout<<"Selection expression corresponding to DSS constructed from `"
			<<expr<<"' is `"<<dssexpr<<"'"<<endl;

		unsigned nComps = dss.components();
		cout<<"The DSS has >>"<<nComps<<"<< components"<<endl;
		for (unsigned n=0; n<nComps; ++n)
			cout<<"Component "<<n<<" has >>"<<dss.component(n)->filters()
				<<"<< filter(s)\n";

		//	see if we get a valid selection expression back
		bool isValid = Evaluator::isValidExpression(dssexpr);
		cout<<"Expression is ... "<<(isValid ? "valid" : "invalid")<<endl;

		return !isValid;
	} catch(SelectException e) {
		errstr<<e.text()<<Sas::error("DssException");
	}

	return 0;
}
