! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program testInterface

  use Dal
  use DSS
  use DataFilter

  type(DataSetT)  :: eset
  type(BlockT)    :: eblock
  type(TableT)    :: etab
  type(DssT)      :: edss, edss2
  type(DScompT)   :: edscomp
  type(DSfilterT) :: edsfilter

  integer :: nSize
  type(RangeT), dimension(:), allocatable :: ranges, ranges0

  character(len=*), parameter :: express = "( RAWX in [0:200],(300:600) && " // &
    "gti(templist.fits:STDGTI,TIME) && region(templist.fits:REGION,X,Y)"     // &
    " && FLAG&0xfff==0 && region(templist.fits:REG00105,X,Y) && TIME in "       // &
    "[0.0000000000000000e+00:1.0000000000000000e+04] ) || ( RAWX in [0:10000] && FLAG&0xfff==0 && region("  // &
    "templist.fits:REG00205,X,Y) && RAWY in [0:200],(300:600) && mask("        // &
    "templist.fits:MASK,0,0,DETX,DETY) ) || ( mask(templist.fits:MASK"       // &
    ",0,0,DETX,DETY) )"

  character(len=255) :: maskSpec

  eset = dataSet("templist.fits", Modify)
  etab = table(eset, "EVENTS")
  eblock = block(etab)

! Read in the DSS

  edss = dataSubspace(eblock)

! Check the selectlib expression

  print *, trim(selectExpression(edss))

  if( trim(selectExpression(edss)) /= express )            &
      call error("SelectExpress", "Expected selection expression not produced")

! Check retrieval of ranges
  call dssSubspaceRangesSize(edss,"RAWX",nSize)
  allocate(ranges(0:nSize-1))
  ranges = dssRanges(edss, "RAWX")
  call dssComponentRangesSize(dssComponent(edss, 0),"RAWX",nSize)
  allocate(ranges0(0:nSize-1))
  ranges0 = dssRanges( dssComponent(edss, 0), "RAWX" )

  if( ranges0(0)%lower%type  /= ranges(0)%lower%type .or.       &
      ranges0(0)%lower%value /= ranges(0)%lower%value  .or.     &
      ranges0(0)%upper%type  /= ranges(0)%upper%type .or.       &
      ranges0(0)%upper%value /= ranges(0)%upper%value  )        &
       call error("RangeRetrieve", "Failed range retrieval test")

! See if filter with a component gives the correct result
  
  call initFilter( selectExpression(dssComponent(edss, 1)) )
  if( doSelect(etab) /= 33 ) call error("Express1", "Failed test expression 1")

! Make sure we can delete stuff

  call dssDeleteComponent(edss, 2)
  call dssDeleteComponent(edss, 0)

  edscomp = dssComponent(edss, 0)

  call dssDeleteFilter(edscomp, 0)
  call dssDeleteFilter(edscomp, 1)

  call initFilter( selectExpression(edss) )  
  if( doSelect(etab) /= 1516 )  &
       call error("Express2", "Failed test expression 2")

! Test the routines for getting filter info

  if( .not. dssHasFilter(edscomp, "DET", MASKFILTERT) ) &
       call error("HasFitler", "Unable to find valid filter.")
  
  edsfilter = dssFilter(edscomp, "DET", MASKFILTERT)

  if( dssFilterType(edsfilter) /= MASKFILTERT ) &
       call error("FilterType", "Retrieved wrong filter type")

  if( dssFilterName(edsfilter) /= "DET" ) &
       call error("FilterName", "Retrieved wrong filter name")

  if( dssFilterUnits(edsfilter) /= "" ) &
       call error("FilterUnits", "Retrieved wrong filter units")

  if( dssFilterComps(edsfilter) /= "DETX,DETY" ) &
       call error("FilterComps", "Retrieved wrong filter components")

  maskSpec = dssFilterBlock(edsfilter)
  if( maskSpec /= "templist.fits:MASK" ) &
       call error("FilterBlock", "Retrieved wrong filter block")

! Clean up

  deallocate(ranges)
  deallocate(ranges0)
  call dssRelease(edss)

! Read the subspace specification back in, and make a new table with it.

  edss = dataSubspace(eblock)
  edss2 = dataSubspace(eblock)

  call dssDeleteComponent(edss, 2)
  call dssDeleteComponent(edss, 1)

  call dssDeleteFilter( dssComponent(edss, 0), 5 )
  call dssDeleteFilter( dssComponent(edss, 0), 4 )
  call dssDeleteFilter( dssComponent(edss, 0), 3 )

  call release(eset) 

  call copyDataSet("emlist.fits","example.fits")

  eset = dataSet("example.fits", Modify)
  etab = table(eset, "EVENTS")
  eblock = block(etab)

  call initFilter( selectExpression(edss) )
  print *, doFilter(etab)
  call dssWrite(edss, eblock)

  call release(eset)

  ! Try merging to subspaces

  call dssMerge(edss, edss2)

  call dssRelease(edss)
  call dssRelease(edss2)

! Open the temporary file, and delete the DSS from it.

  eset = dataSet("templist.fits", Modify)
  etab = table(eset, "EVENTS")
  eblock = block(etab)

  call dssDelete(eblock)

  call release(eset)

end program testInterface
